"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const classnames_1 = require("classnames");
const react_pose_1 = require("react-pose");
const { useEffect, useRef } = React;
const DialogContainer = react_pose_1.default.div({
    preEnter: {
        opacity: 0
    },
    enter: {
        opacity: 1
    },
    exit: {
        opacity: 0,
        delay: 300,
        transition: {
            ease: 'easeIn',
            duration: 250
        }
    }
});
const DialogWrapper = react_pose_1.default.div({
    preEnter: {
        opacity: 0,
        y: '-20px'
    },
    enter: {
        opacity: 1,
        y: '0',
        delay: 250,
        transition: {
            duration: 350
        }
    },
    exit: {
        opacity: 0,
        y: '-20px',
        transition: {
            ease: 'easeIn'
        }
    }
});
exports.Dialog = React.memo(({ children, className, color, contentClassName, onClose, opacity, open, style }) => {
    // Hooks:
    const background = useRef(null);
    useEffect(() => {
        window.addEventListener('keydown', handleKeyDown);
        return () => {
            window.removeEventListener('keydown', handleKeyDown);
        };
    }, []);
    // Handlers:
    const handleKeyDown = (e) => {
        e.keyCode === 27 && onClose();
    };
    return (React.createElement(react_pose_1.PoseGroup, { preEnterPose: "preEnter" }, open && (React.createElement(DialogContainer, { className: classnames_1.default(`Dialog fixed pin overflow-y-auto overflow-x-hidden w-full h-screen flex justify-center items-center`, className && className), key: "DialogContainer" },
        React.createElement("div", { className: classnames_1.default(`Dialog-background pin bg-${color} overflow-y-auto overflow-x-hidden fixed w-full h-screen opacity-${opacity}`), onClick: (e) => e.target === background.current && onClose(), ref: background }),
        React.createElement(DialogWrapper, { className: classnames_1.default('Dialog-content fixed bg-white min-w-6 p-4 focus:outline-none overflow-y-auto', contentClassName && contentClassName), style: style, tabIndex: 0 }, children)))));
});
exports.Dialog.defaultProps = {
    color: 'black',
    opacity: 50
};
