"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Components:
const React = require("react");
const classnames_1 = require("classnames");
const react_pose_1 = require("react-pose");
const createState_1 = require("../../utils/createState");
// ================================================================================================
// Default animations:
const AnimatedList = react_pose_1.default.div({
    preEnter: {
        opacity: 0
    },
    enter: {
        beforeChildren: true,
        opacity: 1,
        staggerChildren: 50
    },
    exit: {
        opacity: 0,
        duration: 250
    }
});
const AnimatedListItem = react_pose_1.default.button({
    preEnter: {
        opacity: 0,
        y: '-20%'
    },
    enter: {
        opacity: 1,
        y: 0
    },
    exit: {
        opacity: 0
    }
});
exports.Textfield = React.memo(props => {
    const { autocomplete, autocompleteFrom, className, classNameInput, id, iconStart, iconEnd, inputProps, label, material, onEnter, onInput, placeholder, style, type, value } = props;
    // Hooks:
    const [state, setState] = createState_1.createState({
        populated: false,
        list: [],
        open: false,
        selIndex: -1
    });
    const listContainer = React.useRef(null);
    // States:
    const { list, populated, open, selIndex } = state;
    // Handlers:
    const handleInput = (e) => {
        handleAutocomplete(e.target.value);
        onInput(e.target.value);
    };
    const handleFocus = () => {
        setState({ populated: true });
    };
    const handleBlur = () => {
        setState({ populated: false, open: false });
    };
    const handleAutocomplete = (val) => {
        if (autocomplete) {
            if (val.length >= autocompleteFrom) {
                const list = autocomplete.filter(v => v.toLowerCase().includes(val.toLowerCase().trim()));
                list.length < 1 ? setState({ open: false }) : setState({ open: true, list, selIndex: -1 });
            }
            else {
                open && setState({ open: false });
            }
        }
    };
    const handleSelect = (val) => {
        onInput(val);
        setState({ open: false });
    };
    const handleKeyDown = (e) => {
        if (list.length > 0) {
            let index;
            switch (e.key) {
                case 'ArrowUp':
                    index =
                        selIndex === -1 ? list.length - 1 : selIndex === 0 ? list.length - 1 : selIndex - 1;
                    setState({ selIndex: index });
                    handleListScroll(index, true);
                    e.preventDefault();
                    break;
                case 'ArrowDown':
                    index = selIndex === -1 ? 0 : selIndex === list.length - 1 ? 0 : selIndex + 1;
                    setState({ selIndex: index });
                    handleListScroll(index, false);
                    e.preventDefault();
                    break;
                case 'Enter':
                    if (open && selIndex > -1) {
                        handleSelect(list[selIndex]);
                        e.preventDefault();
                    }
                    else {
                        onEnter && onEnter();
                    }
                    break;
            }
        }
        else {
            e.key === 'Enter' && onEnter && onEnter();
        }
    };
    const handleListScroll = (index, up) => {
        if (listContainer.current && list.length > 0) {
            const { top: listTop, bottom: listBottom } = listContainer.current.getBoundingClientRect();
            const { top: rowTop, bottom: rowBottom } = listContainer.current.childNodes[index].getBoundingClientRect();
            // Check row inside list container view
            if (rowTop < listTop || rowBottom > listBottom) {
                listContainer.current.childNodes[index].scrollIntoView(up);
            }
        }
    };
    return (React.createElement("div", { className: classnames_1.default(`Textfield relative flex flex-col justify-center`, material && 'material', populated && 'focus', className && className), style: style },
        label && (React.createElement("label", { className: classnames_1.default(`Textfield-label absolute pin-t pointer-events-none transition`, populated || value.length !== 0 ? 'text-xs populated' : 'text-base'), htmlFor: id }, label)),
        React.createElement("div", { className: classnames_1.default(`Textfield-inputWrapper flex w-full h-auto`) },
            iconStart && React.createElement("div", { className: "Textfield-icon-start mr-2" }, iconStart),
            React.createElement("input", Object.assign({ className: classnames_1.default(`Textfield-input w-full relative focus:outline-none`, classNameInput && classNameInput), id: id, onBlur: handleBlur, onFocus: handleFocus, onChange: handleInput, onKeyDown: handleKeyDown, placeholder: !label ? placeholder : populated && value.length === 0 ? placeholder : '', type: type, value: value }, inputProps)),
            iconEnd && React.createElement("div", { className: "Textfield-icon-end ml-auto" }, iconEnd)),
        React.createElement(react_pose_1.PoseGroup, { preEnterPose: "preEnter" }, open && autocomplete && (React.createElement(AnimatedList, { className: "Textfield-list absolute w-full overflow-y-scroll pin-b pin-l z-10", key: "list", ref: listContainer }, list.map((value, i) => {
            const ListItem = i < 6 ? AnimatedListItem : 'button';
            return (React.createElement(ListItem, { key: value, className: classnames_1.default('Textfield-list-item flex items-center w-full px-4 focus:outline-none', list[selIndex] === value && 'active'), onClick: () => handleSelect(value) }, value));
        }))))));
});
exports.Textfield.defaultProps = {
    autocompleteFrom: 3,
    classNameInput: 'bg-transparent',
    type: 'text',
    placeholder: ''
};
